﻿using System.ComponentModel.Composition;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components
{
    [Export]
    [PartCreationPolicy(CreationPolicy.NonShared)]
    public class AddMapServiceComponent : ComponentBase
    {
        #region MapServiceTypes enum

        public enum MapServiceTypes
        {
            Dynamic,
            Tiled
        }

        #endregion

        private GenericDialogViewModel _dialogViewModel;

        public bool ZoomToExtent { get; set; }
        public string MapServiceUrl { get; set; }

        [Dropdown("MapServiceTypes")]
        public MapServiceTypes MapServiceType { get; set; }

        public void AddMapService()
        {
            MapServiceType type = (MapServiceType == MapServiceTypes.Dynamic)
                                      ? Domain.Gis.MapServiceType.Dynamic
                                      : Domain.Gis.MapServiceType.Tiled;
            MapHandler.AddMapservice(MapServiceUrl, type, ZoomToExtent);
            _dialogViewModel.Close();
        }

        protected override string GetIconResourceUri()
        {
            return "Assets/Images/other_tools.png";
        }

        protected override void OnExecute()
        {
            _dialogViewModel = LayoutManager.Instance.ShowViewModel(this,
                                                                    new DialogServiceArgs
                                                                        {HelpId = "tools;" + GetType().FullName});
        }
    }
}