﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Domain;

namespace FDOT.GIS.Client.Components.Commands
{
    [Export(typeof(IScriptableCommand))]
    public class IdentifyCommand : ScriptableCommand<IdentifyCommandParameters>
    {
        public IDictionary<FeatureType, ICollection<Feature>> Result { get; internal set; }

        /// <summary>
        /// A string identifier for this command.
        /// </summary> 
        public override string CommandVerb
        {
            get { return "ident"; }
        }

        /// <summary>
        /// Executes the command with the specified <see cref="IdentifyCommandParameters"/>.
        /// </summary>
        /// <param name="parameters"></param>
        public override void Execute(IdentifyCommandParameters parameters)
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetWorkflow(parameters).GetEnumerator(), executionContext, Complete);
        }

        protected IEnumerable<IResult> GetWorkflow(IdentifyCommandParameters parameters)
        {
            Log.Info("Executing identify for FeatureTypes [{0}].", string.Join(", ", parameters.FeatureTypeIds));

            IEnumerable<IQueryableDataSource> registeredDataSources = VirtualApplicationManager.Current.QueryManager.RegisteredDataSources;

            List<string> featureTypeNames = new List<string>();
                //parameters.FeatureTypeIds.Select(x => x.Split('|')[1].Trim());
            foreach(var f in parameters.FeatureTypeIds)
            {
                var split = f.Split('|');
                if(split != null && split.Count()>1)
                {
                    featureTypeNames.Add(split[1]);
                }
                else
                {
                    featureTypeNames.Add(f);
                }
            }

            List<FeatureType> featureTypes = new List<FeatureType>();
                
                //registeredDataSources.SelectMany(x => 
                                                      //  x.FeatureTypes.Where(y =>
                                                        //    featureTypeNames.Contains(y.Name, StringComparer.CurrentCultureIgnoreCase))).ToList();

            foreach(var d in registeredDataSources)
            {
                foreach(var f in d.FeatureTypes)
                {
                    if(featureTypeNames.Contains(f.Id) || featureTypeNames.Contains(f.Name))
                    {                                 
                        featureTypes.Add(f);
                    }
                }
            }

            var identify = new Identify(parameters.Geometry, featureTypes);
            yield return identify;
            Log.Info("IdentifyCommand found [{0}] Features.", identify.Result.SelectMany(x => x.Value).Count());
            Result = identify.Result;
            //VirtualApplicationManager.Current.QueryManager.ExecuteIdentify(_parameters.Geometry, r =>
            //{
            //    Result = r;
            //    Log.Info("Identify found [{0}] Features.", Result.SelectMany(x => x.Value).Count());
            //}, featureTypes);
            
            
            //yield return null;
        }

        private void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                Failure(e.Error);
            else
                Success(Result);
        }
    }

    public class IdentifyCommandParameters
    {
        public DataContracts.Geometry Geometry { get; set; }
        public string[] FeatureTypeIds { get; set; }
    }
}
