﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Text;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Helpers;
using ICSharpCode.SharpZipLib.BZip2;

namespace FDOT.GIS.Client.Components.Commands
{
    [Export(typeof(IScriptableCommand))]
    public class LoadQueryCommand : ScriptableCommand<CompressedLoadQuery>
    {
        public override string CommandVerb
        {
            get { return "LoadQuery"; }
        }


        public override void Execute(CompressedLoadQuery parameters)
        {
            var component =
                VirtualApplicationManager.Current.Components.SingleOrDefault(
                    c => c.GetType() == typeof(ViewModels.QueryTool.QueryViewModel));
            var query = component as ViewModels.QueryTool.QueryViewModel;
            //.GetComponentByName("QueryTool") as ViewModels.QueryTool.QueryViewModel);
            if (query == null) return;
            query.Execute();

            var value = StringZipHelper.UnzipString(parameters.Compressed);
            var jsonSerialized = JsonSerializerHelper.DeserializeFromJSON<LoadQueryParameters>(value);

            var queryParameters = new QueryParameters
                                      {
                                          GeometryCriteria = jsonSerialized.GeometryCriteria,
                                          QueryCriteria = jsonSerialized.QueryCriteria,
                                          SecondaryCriteria = jsonSerialized.SecondaryCriteria,
                                          FeatureType =
                                              VirtualApplicationManager.Current.QueryManager.RegisteredDataSources.
                                              Single(
                                                  d => d.FeatureTypes.Any(f => f.Id == jsonSerialized.FeatureTypeId)).
                                              FeatureTypes.Single(
                                                  a => a.Id == jsonSerialized.FeatureTypeId)
                                      };
            query.CriteriaViewModel.LoadSpatialQueryOptions(queryParameters);
        }
    }

    public class CompressedLoadQuery
    {
        public string Compressed { get; set; }
    }

    public class LoadQueryParameters
    {
        public string FeatureTypeId { get; set; }
        public GeometryCriteria GeometryCriteria { get; set; }
        public QueryCriteria QueryCriteria { get; set; }
        public ICollection<SecondaryQueryCriteria> SecondaryCriteria { get; set; }
    }

}
