﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;
using System.Linq;

namespace FDOT.GIS.Client.Components.Commands
{
    [Export(typeof(IScriptableCommand))]
    public class QueryCommand : ScriptableCommand<QueryCommandParameters>
    {
        public QueryResult Results { get; private set; }

        public override string CommandVerb
        {
            get { return "query"; }
        }

        public override sealed void Execute(QueryCommandParameters parameters)
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetWorkflow(parameters).GetEnumerator(), executionContext, Complete);
        }

        private void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                Failure(e.Error);
            else
                Success(Results);
        }

        protected IEnumerable<IResult> GetWorkflow(QueryCommandParameters parameters)
        {
            if (!IsInitialized) throw new Exception("Command not initiaized.");

            if (parameters.Options != null && parameters.Options.FeatureType != null)
            {
                MessageViewModel msg=null;

                if (parameters.DisplayResults)
                {
                    msg=new MessageViewModel
                            {
                                Message = "Performing Query",
                                ShowProgress=true
                            };
                    LayoutManager.Instance.ShowMessage(msg, TimeSpan.Zero);
                }

                var queryWorkFlow = new ExecuteQuery(parameters.Options);
                yield return queryWorkFlow;
                Results = queryWorkFlow.Result;

                if(parameters.ZoomToResults)
                {
                    var geometryList=Results.Features.Select(f => f.Geometry);
                    var envelope=GeometryHelper.CombineGeometries(geometryList.ToArray());

                    MapHandler.ZoomToGeometry(envelope);
                }

                if(parameters.DisplayResults)
                {
                    ResultDisplayManager.Instance.DisplayResults(new ResultDisplayArgs(Results)
                                                                     {
                                                                         
                                                                     });
                    LayoutManager.Instance.RemoveMessage(msg);
                }
            }
        }

    }

}