﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.DataContracts;
using QueryResult = FDOT.GIS.Client.Domain.QueryResult;

namespace FDOT.GIS.Client.Components.StartupActions
{
    [Export(typeof(IStartupAction))]
    [PartCreationPolicy(CreationPolicy.NonShared)]
    public class ClientSideLayerStartupAction : IStartupAction
    {
        private string _mapServiceName;
        private string _layerName;
        private string _color;
        private GraphicsManager _graphicsManager;

        public bool IsInitialized
        {
            get;
            private set;
        }

        public string StartupActionText
        {
            get { return "Loading Layers"; }
        }

        protected string MapServiceName
        {
            get { return _mapServiceName; }
        }

        protected string LayerName
        {
            get { return _layerName; }
        }

        protected GraphicsManager GraphicsManager
        {
            get { return _graphicsManager; }
        }

        protected string Name
        {
            get { return _name; }
        }

        protected string Description
        {
            get { return _description; }
        }

        private string _name;
        private string _description;

        public virtual void Initialize(IDictionary<string, string> customSettings)
        {
            if (!customSettings.ContainsKey("MapServiceName") || !customSettings.ContainsKey("LayerName")) return;
            _mapServiceName = customSettings["MapServiceName"];
            _layerName = customSettings["LayerName"];

            if (customSettings.ContainsKey("Color"))
            {
                _color = customSettings["Color"];
            }

            _name = string.Empty;
            if (customSettings.ContainsKey("Name"))
            {
                _name = customSettings["Name"];
            }

            _description = string.Empty;
            if (customSettings.ContainsKey("Description"))
            {
                _description = customSettings["Description"];
            }

            IsInitialized = true;
        }

        public void ExecuteStartupAction()
        {
            var queryParameters = new QueryParameters();

            IQueryableDataSource dataSource = VirtualApplicationManager.Current.QueryManager.GetDataSource(MapServiceName);
            FeatureType feature = dataSource.FeatureTypes.Single(f => f.Name.ToUpper() == LayerName.ToUpper());
            queryParameters.FeatureType = feature;
            queryParameters.QueryCriteria = new QueryCriteria
                                                {
                                                    Attribute = feature.Attributes.Single(a => a.IsIdentifier),
                                                    ComparisonType = ComparisonType.GreaterThan,
                                                    Value = "0"
                                                };

            VirtualApplicationManager.Current.QueryManager.ExecuteQuery(queryParameters, QueryComplete);
        }

        protected virtual void QueryComplete(QueryResult obj)
        {
            if (obj.Features.Count > 0)
            {
                MapHandler.AddDisplayedGraphicsLayer(Name,Description,obj.Features.ToList(),_color);
            }

            ActionFinished();
        }

        protected void ActionFinished()
        {
            Complete(null, null);
        }

        public event EventHandler Complete;
    }
}
