﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Domain;

namespace FDOT.GIS.Client.Components.StartupActions
{
    [Export(typeof(IStartupAction))]
    public class InitialMessagesAction : IStartupAction
    {
        private bool _isInitialized;
        private string _type;
        private string _xapUri;
        private IInitialMessageProvider InitialMessageProvider { get; set; }

        #region IStartupAction Members

        public bool IsInitialized
        {
            get { return _isInitialized; }
        }

        public string StartupActionText
        {
            get { return "Getting InitialMessages."; }
        }

        public void Initialize(IDictionary<string, string> customSettings)
        {
            if (customSettings.ContainsKey("XapUri"))
                _xapUri = customSettings["XapUri"];
            else
                return;

            if (customSettings.ContainsKey("Type"))
                _type = customSettings["Type"];
            else
                return;

            
        }

        public void ExecuteStartupAction()
        {
            Coroutine.BeginExecute(GetLoadXapWorkflow().GetEnumerator(), new ActionExecutionContext(), Completed);
        }

        public event EventHandler Complete;

        #endregion

        private static void Completed(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw new Exception("Error on Loading Xap for InitialMessages", e.Error);
        }

        public IEnumerable<IResult> GetLoadXapWorkflow()
        {
            yield return new ComponentLoader(_xapUri);
            _isInitialized = true;

            InitialMessageProvider =
                VirtualApplicationManager.GetItems<IInitialMessageProvider>().Single(m => m.GetType().FullName == _type);

            InitialMessageProvider.MessagesLoaded += InitialMessageProviderMessagesLoaded;
            InitialMessageProvider.LoadMessages();

            yield break;
        }

        private void InitialMessageProviderMessagesLoaded(object sender, EventArgs e)
        {
            var view = new InitialMessageView();
            var viewModel = new InitialMessageViewModel(InitialMessageProvider.InitialMessages);
            View.SetModel(view, viewModel);
            LayoutManager.Instance.ShowPanel(view,
                                             new DialogServiceArgs
                                                 {Id = "InitialMessage", IsModal = true, Title = "Messages",HasCloseButton=true,Width = 410,HasMinimizeButton=false,HasSettingsButton=false});
            Complete(this, null);
        }
    }
}