﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;

namespace FDOT.GIS.Client.Components.StartupActions
{
    [Export(typeof(IStartupAction))]
    [PartCreationPolicy(CreationPolicy.NonShared)]
    public class MultiColoredClientSideLayerStartupAction : ClientSideLayerStartupAction
    {
        private string _attribute;
        private List<ColorFilter> _colorFilters = new List<ColorFilter>();
        public override void Initialize(IDictionary<string, string> customSettings)
        {
            if (customSettings.ContainsKey("ColorFilters"))
            {
                string[] filters = customSettings["ColorFilters"].Split(';');
                foreach(var f in filters)
                {
                    string[] split = f.Split(',');
                    if(split.Length!=3)
                        throw new Exception("Incorrectly configured MultiColoredClientSideLayerStartupAction.");

                    _colorFilters.Add(new ColorFilter
                                          {
                                              Value=split[0],
                                              HexColor=split[1],
                                              Description=split[2]
                                          });
                }
            }
            if (customSettings.ContainsKey("Attribute"))
            {
                _attribute = customSettings["Attribute"];
            }
            base.Initialize(customSettings);
        }
        protected override void QueryComplete(QueryResult obj)
        {
            if (obj.Features.Count > 0)
            {
                MapHandler.AddDisplayedGraphicsLayer(Name,Description,obj.Features,_colorFilters,_attribute);
            }
            ActionFinished();
        }
    }
}
