﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Persistence;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [DisplayName("Bookmark Extent")]
    public class BookmarkExtentViewModel : ComponentBase
    {
        #region Properties

        public List<string> SavedExtents
        {
            get { return StateManager.SavedExtents.Keys.ToList(); }
        }

        #endregion

        private string _extentName;

        public string ExtentName
        {
            get { return _extentName; }
            private set { _extentName = value; NotifyOfPropertyChange(() => ExtentName); }
        }

        #region Protected/Public Methods
        protected override string IconResourceUri
        {
            get { return "/FDOT.GIS.Client;component/Assets/Images/lightning.png"; }
        }

        public void SaveExtent(object sender, EventArgs e)
        {
            Debug.WriteLine("Save: " + (sender as FrameworkElement).DataContext);
            string key = GetKey(sender);
            if (!string.IsNullOrEmpty(key) && !SavedExtents.Any(x => x.Equals(key, StringComparison.CurrentCultureIgnoreCase)))
            {
                StateManager.SaveExtent(key, MapHandler.Extent);
                NotifyOfPropertyChange(() => SavedExtents);
                ExtentName = string.Empty;
            }
        }

        public void DeleteExtent(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            Debug.WriteLine("Delete: " + (sender as FrameworkElement).DataContext);
            string key = GetKey(sender);
            if (SavedExtents.Any(x => x.Equals(key, StringComparison.CurrentCultureIgnoreCase)))
            {
                StateManager.SavedExtents.Remove(key);
                NotifyOfPropertyChange(() => SavedExtents);
            }
        }

        public void SelectExtent(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            string key = GetKey(sender);
            if (SavedExtents.Any(x => x.Equals(key, StringComparison.CurrentCultureIgnoreCase)))
            { MapHandler.Extent = StateManager.SavedExtents[key]; }
        }
        #endregion

        private string GetKey(object sender)
        {
            var frameworkElement = sender as FrameworkElement;
            if (frameworkElement != null)
            {
                object dataContext = frameworkElement.DataContext;
                if (dataContext != null)
                { return dataContext.ToString(); }
            }
            return string.Empty;
        }


        #region component implementation
        [Hidden]
        public override DependencyObject HelpView
        {
            get
            {
                var view= new BookmarkExtentHelpView();
                var viewModel = new BookmarkExtentHelpViewModel();
                View.SetModel(view,viewModel);
                return view;
            }
        }

        private GenericDialogViewModel _viewModel;
        protected override void OnExecute()
        {
            var view = new BookmarkExtentView();
            View.SetModel(view, this);

            _viewModel = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
            {
                Title = this.ComponentConfiguration.Name,
                HelpId = "tools;" + GetType().FullName,
                Icon = Icon
            });

            _viewModel.Closed += OnClose;
        }

        private void OnClose(object sender, EventArgs e)
        { _viewModel = null; }

        protected override void OnInitialize(IDictionary<string, string> customSettings) { }

        #endregion
    }

}
