﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    public class IdentifyResultAttributesViewModel : PropertyChangedBase
    {
        private readonly GraphicsManager _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
        private readonly IdentifyResultAttributesView _view;

        public IdentifyResultAttributesViewModel() { }

        public IdentifyResultAttributesViewModel(IdentifyResultAttributesView view)
        {
            _view = view;
        }

        private ObservableCollection<Feature> _identifyResults = new ObservableCollection<Feature>();
        public ObservableCollection<Feature> IdentifyResults
        {
            get { return _identifyResults; }
        }

        private DependencyObject AttributesGrid
        {
            get
            {
                return _view.DescendentsOfType<DataGrid>().First(d => ((FrameworkElement)d).Name == "AttributesGrid");
            }
        }

        private GenericDialogViewModel _dialog;
        public GenericDialogViewModel Dialog
        {
            get { return _dialog; }
            set
            {
                _dialog = value;
                _dialog.Closed += _dialog_Closed;
            }
        }

        void _dialog_Closed(object sender, EventArgs e)
        {
            _dialog.Closed -= _dialog_Closed;
            _view.Content = null;
            _view.DataContext = null;

            _dialog = null;
            _identifyResults.Clear();
            _identifyResults = null;
            _graphicsManager.ClearGraphics();
            _graphicsManager.Dispose();
        }


        public void ItemSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            _graphicsManager.ClearGraphics();
            var feature = ((ListBox)sender).SelectedValue as Feature;
            if (feature != null)
            {
                _graphicsManager.AddGraphic(feature, null, null);
                var executionContext = new ActionExecutionContext();
                FeatureActionHelper.DetachContextMenu(AttributesGrid);
                Coroutine.BeginExecute(FeatureActionHelper.AttachContextMenu(AttributesGrid, feature).GetEnumerator(), executionContext, Complete);
            }
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        public void SendToResultsView(object sender, RoutedEventArgs e)
        {
            var item = sender as MenuItem;

            // If the DataContext of the item is a string, we'll assume it's the name of the list the context menu goes with
            // this is a workaround for the contextmenu not having any accessible relationship to the listbox that contains it.
            // Attempts to bind the menu to the selected items or value property on its corresponding list only resulted in all of
            // items showing up in the menu's data context, again with no path back to the list.
            if (item != null && item.DataContext != null)
            {
                ListBox parentList = _view.Descendents().OfType<ListBox>().Where(x => x.Name.Equals(item.DataContext.ToString(), StringComparison.CurrentCultureIgnoreCase)).FirstOrDefault();
                if (parentList != null)
                {
                    var results = new List<Feature>();
                    foreach (var selectedItem in parentList.SelectedItems)
                    {
                        if (selectedItem is Feature)
                        { results.Add((Feature)selectedItem); }
                        else if (selectedItem is KeyValuePair<string, List<Feature>>)
                        { results.AddRange(((KeyValuePair<string, List<Feature>>)selectedItem).Value); }
                    }

                    var executionContext = new ActionExecutionContext();

                    //TODO: fix this, should have to hack around to get featureType
                    if (results.Count > 0)
                    {
                        FeatureType featureType = results.First().FeatureType;
                        Coroutine.BeginExecute(SendToResultsViewWorkflow(featureType, results).GetEnumerator(), executionContext, Complete);
                    }
                }
            }
        }

        private IEnumerable<IResult> SendToResultsViewWorkflow(FeatureType featureType, ICollection<Feature> features)
        {
            ResultDisplayArgs args = GetResultDisplayArgs(featureType, features);
            ResultDisplayManager.Instance.DisplayResults(args);
            yield break;
        }

        private ResultDisplayArgs GetResultDisplayArgs(FeatureType featureType, ICollection<Feature> features)
        {
            var args = new ResultDisplayArgs(new QueryResult(featureType, new Core.ObservableCollection<Feature>(features)))
                           {
                               Title = features.Count == 1
                                           ? "Identify : " + features.Single()
                                           : "Identify : " + featureType.Name,
                               MapTipColumn = featureType.Attributes.Single(a => a.IsIdentifier).Name,
                               MapTipLabel = featureType.Attributes.Single(a => a.IsIdentifier).Name
                           };

            return args;
        }

    }
}
