﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class IdentifyViewModel : ComponentBase
    {
        private GeometrySelectionService _geometrySelectionService;

        public override DependencyObject HelpView
        {
            get
            {
                var help = new IdentifyHelpView();
                View.SetModel(help, new IdentifyHelpViewModel());
                return help;
            }
        }


        protected override string IconResourceUri
        {
            get { return "Assets/Images/identify.png"; }
        }


        protected override void OnInitialize(IDictionary<string, string> customSettings)
        {
            var parameters = new GeometrySelectionServiceConfiguration { Color = Colors.Red };
            _geometrySelectionService = new GeometrySelectionService(parameters);
        }


        protected override void OnExecute()
        {
            ToolbarViewModel.Instance.ToolbarItemClicked += MainToolbarItemClicked;
            _geometrySelectionService.DrawCanceled += DrawCanceled;
            _geometrySelectionService.SelectGeometry(GeometryType.Point, GeometrySelectionComplete);
        }


        private void DrawCanceled(object sender, EventArgs e)
        {
            DeactivateComponent();
        }


        private void MainToolbarItemClicked(ToolbarItem item)
        {
            if (item.Component == this)
                DeactivateComponent();
        }


        private void GeometrySelectionComplete(DataContracts.Geometry geometry)
        {
            if (_geometrySelectionService != null)
            {
                _geometrySelectionService.DrawCanceled -= DrawCanceled;
                var executionContext = new ActionExecutionContext();
                Coroutine.BeginExecute(GetResultsBufferWorkflow(geometry).GetEnumerator(), executionContext, Completed);
            }
        }

        private void Completed(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw new Exception("IdentifyWorflowException", e.Error);
        }


        private IEnumerable<IResult> GetResultsBufferWorkflow(DataContracts.Geometry geometry)
        {
            DeactivateComponent();

            // drop image on map
            BitmapImage image = ResourceHelper.GetBitmap("Assets/Images/identify-map-symbol.png");
            GraphicsManager graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
            graphicsManager.AddGraphic(geometry, new SymbolDefinition(image), null, null);

            // show message
            var message = new MessageViewModel { ShowProgress = true, Message = "Executing Identify" };
            LayoutManager.Instance.ShowMessage(message, TimeSpan.Zero);

            var workflow = new IdentifyWorkflow(geometry);
            yield return workflow;

            //var viewModel = new IdentifyResultsViewModel();
            var view = new IdentifyResultAttributesView();
            var viewModel = new IdentifyResultAttributesViewModel(view);
            if (workflow.Result != null && workflow.Result.Count() > 0)
            {
                foreach (var kvp in workflow.Result)
                {
                    foreach (var feature in kvp.Value)
                        viewModel.IdentifyResults.Add(feature);
                }
            }
            View.SetModel(view, viewModel);
            //var view = new IdentifyResultAttributesView() { DataContext = viewModel };
            GenericDialogViewModel result = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
            {
                Id = Guid.NewGuid().ToString(),
                Title = ComponentConfiguration.Name,
                HelpId = "tools;" + GetType().FullName,
                Icon = Icon,
                HasMinimizeButton = false
            });
            viewModel.Dialog = result;
            //var viewModel = new IdentifyResultsViewModel();
            //if (workflow.Result != null && workflow.Result.Count() > 0)
            //{
            //    foreach (var kvp in workflow.Result)
            //    {
            //        foreach (var feature in kvp.Value)
            //            viewModel.IdentifyResults.Add(feature);
            //    }
            //}

            //var view = new IdentifyView { DataContext = viewModel };
            //var result = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs { Id = Guid.NewGuid().ToString(), Title = ComponentConfiguration.Name, HelpId = "tools;" + GetType().FullName, Icon = Icon });

            result.Closed += (s, e) => { graphicsManager.ClearGraphics(); graphicsManager.Dispose(); };
            LayoutManager.Instance.RemoveMessage(message);
        }

        private void DeactivateComponent()
        {
            ToolbarViewModel.Instance.ToolbarItemClicked -= MainToolbarItemClicked;
            ToolbarViewModel.Instance.InactivateComponent(typeof(IdentifyViewModel));
            _geometrySelectionService.DrawCanceled -= DrawCanceled;
        }
    }
}
