﻿using System;
using System.Collections.Generic;
using System.Linq;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;

namespace FDOT.GIS.Client.Components.ViewModels
{
    public class IdentifyWorkflow : ResultBase
    {
        private readonly IdentifyWorkflowParameters _parameters;
        public IdentifyWorkflow(DataContracts.Geometry geometry, IdentifyWorkflowParameters parameters)
        {
            _geometry = geometry;
            this._parameters = parameters;
        }

        public IDictionary<FeatureType, ICollection<Feature>> Result { get; internal set; }

        public IdentifyWorkflow(DataContracts.Geometry geometry)
        {
            _geometry = geometry;
        }

        private readonly DataContracts.Geometry _geometry;

        public override void Execute(ActionExecutionContext context)
        {
            try
            {
                if (_parameters == null)
                {
                    VirtualApplicationManager.Current.QueryManager.ExecuteIdentify(_geometry, r => { Result = r; Success(); }, null);
                }
                else
                {
                    IQueryableDataSource dataSource = VirtualApplicationManager.Current.QueryManager.GetDataSource(_parameters.MapServiceName);
                    IEnumerable<FeatureType> featureTypes = _parameters.LayerIds.Select(ds => dataSource.FeatureTypes.Single(ft => ft.Id.Equals(dataSource.Id + "|" + ds)));
                    dataSource.Identify(_geometry, r => { Result = r; Success(); }, featureTypes);
                }
            }
            catch (Exception ex)
            {
                Failure(ex);
            }
        }
    }
}