﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Browser;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Components.Views.Printing;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.PrintMap;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.Printing
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class PrintOptionsViewModel : ComponentBase
    {
        public string Title { get; set; }
        public bool ShowLegend { get; set; }
        private GenericDialogViewModel _dialogViewModel;

        private bool isBusy;
        public bool IsBusy
        {
            get { return isBusy; }
            set
            {
                isBusy = value;
                NotifyOfPropertyChange(() => IsBusy);
            }
        }

        public void PrintPreview()
        {
            IsBusy = true;
            var options = new PrintMapOptions();
            var mapServices = new List<MapServiceLayer>();

            options.Title = Title;
            options.ShowLegend = ShowLegend;

            if (MapHandler.BaseMap != null && MapHandler.BaseMap.Visible)
                mapServices.Add(new MapServiceLayer { Opacity = MapHandler.BaseMap.Opacity, Url = MapHandler.BaseMap.Url });

            foreach (var l in MapHandler.Layers.OfType<DynamicMapService>())
            {
                var mapService = l;
                if (!mapService.VisibleLayerIds.Any()) continue;

                var layers = mapService.Layers.Where(lyr => mapService.VisibleLayerIds.Contains(lyr.Id));

                var layer = new MapServiceLayer
                                {
                                    Url = mapService.Url,
                                    Opacity = mapService.Opacity,
                                    VisibleLayerIds = new ObservableCollection<int>(layers.Where(lyr => lyr.IsEnabled && lyr.IsVisibleAtCurrentExtent).Select(lyr => lyr.Id))
                                };

                mapServices.Add(layer);
            }
            options.MapServiceLayers = new ObservableCollection<MapServiceLayer>(mapServices);
            options.ViewExtent = GeometryTranslator.TranslateMapEnvelope(MapHandler.GetExtent());
            options.BoundingBoxSpatialReference = MapHandler.MapSpatialReference;

            //TODO: Work on this for printing map graphics
            //var bitmaps = new List<WriteableBitmap>();


            //WriteableBitmap bottomBitmap = null;// MapHandler.CurrentMap.Layers[0].ToBitmap(); ;
            //var graphicsList = MapHandler.CurrentMap.Layers.Where(l => l.GetType() == typeof(GraphicsLayer)).ToList();

            //for (int index = 0; index < graphicsList.Count; index++)
            //{
            //    var l = graphicsList[index];

            //    if (index == 0)
            //    {
            //        bottomBitmap = l.ToBitmap();
            //        continue;
            //    }

            //    var topBitmap = l.ToBitmap();
            //    bottomBitmap.Blit(new Rect { Height = bottomBitmap.PixelHeight, Width = bottomBitmap.PixelWidth }, topBitmap,
            //                new Rect { Height = bottomBitmap.PixelHeight, Width = bottomBitmap.PixelWidth });
            //}

            //for (var i = 1; i < MapHandler.CurrentMap.Layers.Count - 1; i++)
            //{
            //    var bitmap = MapHandler.CurrentMap.Layers[i].ToBitmap();
            //    bottomBitmap.Blit(new Rect { Height = bitmap.PixelHeight, Width = bitmap.PixelWidth }, bitmap, new Rect { Height = bitmap.PixelHeight, Width = bitmap.PixelWidth });
            //}

            //options.GraphicsLayer = bottomBitmap.ToPng();

            var svs = ServiceClients.GetPrintMapServiceClient();
            svs.GetPrintMapPdfCompleted += svs_GetPrintMapPdfCompleted;
            svs.GetPrintMapPdfAsync(options);
        }

        void svs_GetPrintMapPdfCompleted(object sender, GetPrintMapPdfCompletedEventArgs e)
        {
            HtmlPage.Window.Navigate(new Uri(VirtualApplicationManager.HostSiteUrl + "Printing/PrintMap.ashx?id=" + e.Result));
            IsBusy = false;
            _dialogViewModel.Close();
        }

        private List<string> _urls = new List<string>();

        protected override void OnExecute()
        {
            var view = new PrintOptionsView();
            Caliburn.Micro.View.SetModel(view, this);
            _dialogViewModel = LayoutManager.Instance.ShowPanel(view, new DialogServiceArgs
            {
                Title = "Print Options",
                HelpId = "tools;" + GetType().FullName,
                Icon = Icon
            });
        }

        public override DependencyObject HelpView
        {
            get
            {
                var help = new PrintHelpView();
                View.SetModel(help, new PrintHelpViewModel());
                return help;
            }
        }

        protected override string GetIconResourceUri()
        {
            return "Assets/Images/print.png";
        }
    }
}
