﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Controls;
using Caliburn.Micro;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.DotCodes;
using FDOT.GIS.DataContracts;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    [Export("QueryCriterionViewModel", typeof (QueryCriterionViewModel))]
    public class QueryCriterionViewModel : PropertyChangedBase
    {
        private QueryCriteria _criteria;
        private ObservableCollection<DotCode> _criteriaCodes;
        private bool _isFreeTextCriterion;
        private bool _isDateTimeCriterion;
        private Layer _layer;
        private FeatureType _featureType;

        public Layer Layer
        {
            get { return _layer; }
            set
            {
                _layer = value;
                NotifyOfPropertyChange(() => Layer);
            }
        }

        private IEnumerable<string> _searchResults;
        public IEnumerable<string> SearchResults
        {
            get { return _searchResults; }
            set { _searchResults = value; NotifyOfPropertyChange(()=>SearchResults);}
        }

        public FeatureType FeatureType
        {
            get { return _featureType; }
            set
            {
                _featureType = value;
                NotifyOfPropertyChange(() => FeatureType);
            }
        }


        public QueryCriteria Criteria
        {
            get { return _criteria; }
            set
            {
                _criteria = value;
                NotifyOfPropertyChange(() => Criteria);
            }
        }

        public ObservableCollection<DotCode> CriteriaCodes
        {
            get { return _criteriaCodes; }
            set
            {
                if (value != null && value.Count == 0)
                {
                    _criteriaCodes = null;
                }
                else
                {
                    _criteriaCodes = value;
                }
                IsFreeTextCriterion = _criteriaCodes == null;
                NotifyOfPropertyChange(() => CriteriaCodes);
            }
        }


        public bool IsFreeTextCriterion
        {
            get { return _isFreeTextCriterion; }
            set
            {
                _isFreeTextCriterion = value;
                NotifyOfPropertyChange(() => IsFreeTextCriterion);
            }
        }


        public bool IsDateTimeCriterion
        {
            get { return _isDateTimeCriterion; }
            set
            {
                _isDateTimeCriterion = value;
                if (_isDateTimeCriterion)
                { IsFreeTextCriterion = false; }
                NotifyOfPropertyChange(() => IsDateTimeCriterion);
            }
        }

        public void Initialize(FeatureType featureType, QueryCriteria criteria)
        {
            FeatureType = featureType;
            Criteria = criteria;
        }


        public void CriteriaSelectionChanged(object sender, RoutedEventArgs e)
        {
            if (Criteria != null && Criteria.Attribute != null && !string.IsNullOrEmpty(Criteria.Attribute.Name))
            {
                if (_criteria != null && _criteria.Attribute != null)
                { IsDateTimeCriterion = _criteria.Attribute.Type == typeof(DateTime); }

                if (!IsDateTimeCriterion)
                { Coroutine.BeginExecute(DoWorkFlow().GetEnumerator(), null, Complete); }
            }
        }

        public void SelectionChanged(object sender, RoutedEventArgs e)
        {
            EventAggregatorManager.Events.Publish(new CriterionChangedEvent(Criteria));
        }

        public void CriterionChanged(object sender, RoutedEventArgs e)
        {
            string term = ((AutoCompleteBox) sender).Text;
            var layer=MapHandler.GetLayerForFeatureType(this.FeatureType);
            //layer.Layer
            if (layer.ParentMapService.UseOracleQueries && term.Length>1 && this.Criteria.Attribute.Type == typeof(string))
            {
                var svs = ServiceClients.QueryServiceClient;
                svs.AutocompleteCompleted += QueryServiceClient_AutocompleteCompleted;
                svs.AutocompleteAsync(term, Criteria.Attribute.Name, layer.Layer, layer.ParentMapService.Url);
            }

            UpdateSource(sender);
            EventAggregatorManager.Events.Publish(new CriterionChangedEvent(Criteria));
        }

        void QueryServiceClient_AutocompleteCompleted(object sender, Query.AutocompleteCompletedEventArgs e)
        {
            this.SearchResults = e.Result;
        }


        public static void UpdateSource(object sender)
        {
            var textBox = sender as AutoCompleteBox;

                        
            //if (textBox != null)
            //{
            //    // If a TextBox has the focus, this forces it to update its source.
            //    // It is needed when using Enter and Escape to activate Save and Cancel button,
            //    // because the focused TextBox does not update in Silverlight, even when
            //    // a call is made to ButtonSave.Focus() or ButtonCancel.Focus().
            //    var tb = System.Windows.Input.FocusManager.GetFocusedElement() as TextBox;
            //    if (tb != null)
            //        tb.GetBindingExpression(TextBox.TextProperty).UpdateSource();
            //}
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private IEnumerable<IResult> DoWorkFlow()
        {
            yield return CodeProviderManager.Instance.GetProvider(Criteria.Attribute, this, "CriteriaCodes");
        }
    }

    public class CriterionChangedEvent
    {
        public QueryCriteria Criteria { get; private set; }

        public CriterionChangedEvent(QueryCriteria criteria)
        {
            Criteria = criteria;
        }
    }

}