﻿using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using FDOT.GIS.Client.Components.Commands;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Helpers;
using FDOT.GIS.Client.ViewModels;
using FDOT.GIS.DataContracts;
using ICSharpCode.SharpZipLib.BZip2;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    public class QueryShareViewModel : Core.ObservableBase, IDataErrorInfo
    {
        private string _toEmailAddress;
        [Required(ErrorMessage = "Error: ToEmailAddress is required.")]
        [RegularExpression(@"^(?("")(""[^""]+?""@)|(([0-9a-zA-Z]((\.(?!\.))|[-!#\$%&'\*\+/=\?\^`\{\}\|~\w])*)(?<=[0-9a-zA-Z])@))" +
              @"(?(\[)(\[(\d{1,3}\.){3}\d{1,3}\])|(([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,6}))$", ErrorMessage = "Error: Not a valid email address.")]
        public string ToEmailAddress
        {
            get { return _toEmailAddress; }
            set
            {
                if (value == _toEmailAddress) return;
                Validator.ValidateProperty(value, new ValidationContext(this, null, null) { MemberName = "ToEmailAddress" });

                _toEmailAddress = value;
                NotifyOfPropertyChange(() => ToEmailAddress);
            }
        }

        private string _from;
        [Required(ErrorMessage = "Error: From is required.")]
        public string From
        {
            get { return _from; }
            set
            {
                if (value == _from) return;
                Validator.ValidateProperty(value, new ValidationContext(this, null, null) { MemberName = "From" });

                _from = value;
                NotifyOfPropertyChange(() => From);
            }
        }

        private string _message;
        [Required(ErrorMessage = "Error: Message is required.")]
        public string Message
        {
            get { return _message; }
            set
            {
                if (value == _message) return;

                Validator.ValidateProperty(value, new ValidationContext(this, null, null) { MemberName = "Message" });
                _message = value;
                NotifyOfPropertyChange(() => Message);

            }
        }

        public GenericDialogViewModel Dialog { get; set; }

        public QueryParameters QueryParameters { get; set; }

        public void ShareQuery()
        {
            if (Validator.TryValidateObject(this, new ValidationContext(this, null, null), null, true))
            {
                var svs = ServiceClients.QueryServiceClient;
                var queryParams = new LoadQueryParameters
                                       {
                                           FeatureTypeId = QueryParameters.FeatureType.Id,
                                           GeometryCriteria = QueryParameters.GeometryCriteria,
                                           QueryCriteria = QueryParameters.QueryCriteria,
                                           SecondaryCriteria = QueryParameters.SecondaryCriteria
                                       };
                var jsonSerialized = JsonSerializerHelper.SerializeToJSON(queryParams, false);

                var value = StringZipHelper.ZipString(jsonSerialized);
                string url = VirtualApplicationManager.HostSiteUrl.ToString();
                url += "?AppId=" + VirtualApplicationManager.CurrentApplicationId + "&LoadQuery={'Compressed':'" +
                       System.Windows.Browser.HttpUtility.UrlEncode(value) + "'}";

                var options = new ShareQueryOptions
                                  {
                                      ApplicationAcronym = VirtualApplicationManager.CurrentApplicationId,
                                      ApplicationName = VirtualApplicationManager.Current.Configuration.Title,
                                      From = From,
                                      LinkUrl = url,
                                      Message = Message,

                                      FromEmailAddress = "",
                                      ToEmailAddress = ToEmailAddress
                                  };
                options.Hash = options.GetHash();

                svs.ShareQueryLinkAsync(options);
                Dialog.Close();
            }
        }



        public void Cancel()
        {
            Dialog.Close();
        }

        public string this[string columnName]
        {
            get { return null; }
        }

        public string Error
        {
            get { return null; }
        }
    }
}
