﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Components.Views.QueryTool;
using FDOT.GIS.Client.Configuration;
using FDOT.GIS.Client.Core;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Persistence;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    [Export]
    [PartCreationPolicy(CreationPolicy.NonShared)]
    public class QueryViewModel : Conductor<Screen>, IComponentBase
    {
        private ICommand _switchToNew;
        public ICommand SwitchToNew
        {
            get { return _switchToNew; }
            set
            {
                _switchToNew = value;
                NotifyOfPropertyChange(() => SwitchToNew);
            }
        }

        private ICommand _switchToSaved;
        public ICommand SwitchToSaved
        {
            get { return _switchToSaved; }
            set
            {
                _switchToSaved = value;
                NotifyOfPropertyChange(() => SwitchToSaved);
            }
        }

        private ICommand _switchToRecent;
        public ICommand SwitchToRecent
        {
            get { return _switchToRecent; }
            set
            {
                _switchToRecent = value;
                NotifyOfPropertyChange(() => SwitchToRecent);
            }
        }

        private QueryCriteriaViewModel _criteriaViewModel;
        public GenericDialogViewModel Dialog { get; set; }

        private UserControl _criteriaView;
        public UserControl CriteriaView
        {
            get { return _criteriaView; }
            internal set
            {
                _criteriaView = value;
                NotifyOfPropertyChange(() => CriteriaView);
            }
        }


        public bool RecentItemsEnabled
        {
            get { return StateManager.DoPersistRecentItems; }
        }

        public bool QuerySaveEnabled
        {
            get { return StateManager.DoPersistRecentQueries; }
        }


        public string QuerySection { get; set; }

        public void Initialize(IDictionary<string, string> customSettings)
        {

        }

        // ReSharper disable UnusedAutoPropertyAccessor.Local
        public bool IsExecuting { get; private set; }
        // ReSharper restore UnusedAutoPropertyAccessor.Local

        public BitmapImage Icon
        {
            get { return ResourceHelper.GetBitmap("Assets/Images/query.png"); }
        }

        public ApplicationComponentConfiguration ComponentConfiguration { get; set; }

        public void Execute()
        {
            queryView = new QueryView();
            View.SetModel(queryView, this);
            Dialog = LayoutManager.Instance.ShowPanel(queryView,
                                                      new DialogServiceArgs
                                                      {
                                                          Id = "Query",
                                                          Title = ComponentConfiguration.Name,
                                                          HelpId = "tools;" + GetType().FullName,
                                                          Icon = Icon,
                                                          ShowSettingsOnBack = true,
                                                          SettingsViewModel = SettingsViewModel
                                                      });


            var view = new QueryCriteriaView();
            _criteriaViewModel = new QueryCriteriaViewModel(Dialog);

            Dialog.Closed += OnDialogOnClosed;

            View.SetModel(view, CriteriaViewModel);
            CriteriaView = view;

            QuerySection = "New";

            SwitchToNew = new DelegateCommand(o =>
            {
                QuerySection = "New";
                ChangeActiveItem(null, true);
                _criteriaViewModel.Clear();
                IsLeftBorderVisibility = false;
            });

            SwitchToSaved = new DelegateCommand(o =>
            {
                QuerySection = "Saved";
                var vm = new SavedQueryViewModel();
                var v = new SavedQueryView();
                View.SetModel(v, vm);
                ChangeActiveItem(vm, true);
                _criteriaViewModel.Clear();
                IsLeftBorderVisibility = true;
            });

            SwitchToRecent = new DelegateCommand(o =>
            {
                QuerySection = "Recent";
                var vm = new RecentQueryViewModel();
                var v = new RecentQueryView();
                View.SetModel(v, vm);
                _criteriaViewModel.Clear();
                ChangeActiveItem(vm, true);
                IsLeftBorderVisibility = true;
            });
            SwitchToNew.Execute(null);
        }

        private void OnDialogOnClosed(object sender, EventArgs args)
        {
            _criteriaViewModel.Clear();
            _criteriaViewModel = null;

            Dialog.Closed -= OnDialogOnClosed;
            Dialog = null;

            SwitchToNew = null;
            SwitchToRecent = null;
            SwitchToSaved = null;

            CriteriaView.DataContext = null;
            CriteriaView = null;

            ClearViews();
            queryView.DataContext = null;
            queryView = null;    
        }

        public QuerySettingsViewModel SettingsViewModel
        {
            get { return QuerySettingsViewModel.Instance; }
        }


        public DependencyObject HelpView
        {
            get
            {
                var viewModel = new Help.QueryToolHelpViewModel();
                var view = new QueryToolHelpView();
                View.SetModel(view, viewModel);

                return view;
            }
        }

        private bool _isLeftBorderVisibility;
        private QueryView queryView;

        public bool IsLeftBorderVisibility
        {
            get { return _isLeftBorderVisibility; }
            set
            {
                if (_isLeftBorderVisibility == value) return;
                _isLeftBorderVisibility = value;
                NotifyOfPropertyChange(() => IsLeftBorderVisibility);
            }
        }

        public QueryCriteriaViewModel CriteriaViewModel
        {
            get { return _criteriaViewModel; }
        }
    }
}
