﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using Caliburn.Micro;
using FDOT.GIS.Client.Controls;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Persistence;

namespace FDOT.GIS.Client.Components.ViewModels.QueryTool
{
    public class RecentQueryViewModel : Screen
    {
        public ICollectionView SavedQueries
        {
            get
            {
                var groupedItems = new List<GroupedItem>();
                StateManager.RecentQueries.ToList().ForEach(x => groupedItems.Add(new GroupedItem { DisplayText = ConvertCriteriaToString(x), Group = x.FeatureType.Name, Value = x }));

                var view = new PagedCollectionView(groupedItems);
                view.GroupDescriptions.Add(new PropertyGroupDescription("Group"));
                return view;
            }
        }

        private static string ConvertCriteriaToString(QueryParameters options)
        {
            var criteriaStrings = new List<string>();
            if (options != null)
            {
                criteriaStrings.Add(ConvertCriterionToString(options.QueryCriteria));

                if (options.SecondaryCriteria != null)
                { options.SecondaryCriteria.ToList().ForEach(x => criteriaStrings.Add(ConvertCriterionToString(x))); }
            }
            string value = string.Join(", ", criteriaStrings.Where(x => x.Trim() != string.Empty));
            return string.IsNullOrEmpty(value) ? "(no text criteria - click to preview on map)" : value;
        }


        private static string ConvertCriterionToString(QueryCriteria criterion)
        {
            if (criterion != null)
            {
                // we should've already preloaded the codes needed for saved queries, so they should cached already
                CodeProviderBase codeProvider = CodeProviderManager.Instance.GetProvider(criterion.Attribute.Name);
                string codeText = codeProvider.GetDescription(criterion.Attribute.Name, criterion.Value).Trim();
                codeText = string.IsNullOrEmpty(codeText) ? criterion.Value : codeText;

                return string.Format("{0} {1} \"{2}\"", criterion.Attribute.Description,
                                                        DataContracts.ComparisonTypeConverter.ToOperatorString(criterion.ComparisonType),
                                                        codeText);
            }
            return string.Empty;
        }

        public void SavedQuerySelected(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var item = e.NewValue as GroupedItem;
            if (item == null) return;

            var queryOptions = item.Value as QueryParameters;
            if (queryOptions != null)
                ((QueryViewModel)Parent).CriteriaViewModel.LoadSpatialQueryOptions(queryOptions);
        }
    }
}
