﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Windows.Media.Imaging;
using Caliburn.Micro;
using FDOT.GIS.Client.Core;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.ViewModels;
using FDOT.GIS.DataContracts;

namespace FDOT.GIS.Client.Components.ViewModels.Toc
{
    public class AddLayerViewModel : ObservableBase
    {
        public GenericDialogViewModel Dialog { get; set; }

        private string _url;
        public string Url
        {
            get { return _url; }
            set
            {
                if (_url == value)
                    return;
                _url = value;
                NotifyOfPropertyChange(() => Url);
            }
        }

        private bool _isBasemap;
        public bool IsBasemap
        {
            get { return _isBasemap; }
            set { _isBasemap = value;
                NotifyOfPropertyChange(()=>IsBasemapVisible); }
        }

        private bool _isBasemapVisible;
        public bool IsBasemapVisible
        {
            get { return _isBasemapVisible; }
            set { _isBasemapVisible = value; NotifyOfPropertyChange(() => IsBasemapVisible); }
        }

        private bool _saveLayer;
        public bool SaveLayer
        {
            get { return _saveLayer; }
            set { _saveLayer = value;NotifyOfPropertyChange(()=>SaveLayer); }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
            set
            {
                if (_description == value)
                    return;
                _description = value;
                NotifyOfPropertyChange(() => Description);
            }
        }

        private string _copyrightText;
        public string CopyrightText
        {
            get { return _copyrightText; }
            set
            {
                if (_copyrightText == value)
                    return;
                _copyrightText = value;
                NotifyOfPropertyChange(() => CopyrightText);
            }
        }

        private string _title;
        public string Title
        {
            get { return _title; }
            set
            {
                if (_title == value)
                    return;
                _title = value;
                NotifyOfPropertyChange(() => Title);
            }
        }

        private bool _isImageVisible;
        public bool IsImageVisible
        {
            get { return _isImageVisible; }
            set
            {
                _isImageVisible = value;
                NotifyOfPropertyChange(() => IsImageVisible);
            }
        }

        private bool _showMapServiceNotFound;
        public bool ShowMapServiceNotFound
        {
            get { return _showMapServiceNotFound; }
            set { _showMapServiceNotFound = value; NotifyOfPropertyChange(() => ShowMapServiceNotFound); }
        }

        private bool _isTiledMapServiceInCorrectProjection=true;
        public bool IsTiledMapServiceInCorrectProjection
        {
            get { return _isTiledMapServiceInCorrectProjection; }
            set { _isTiledMapServiceInCorrectProjection = value; NotifyOfPropertyChange(() => IsTiledMapServiceInCorrectProjection); }
        }

        public void LoadMapService()
        {
            Coroutine.BeginExecute(GetMapServiceWorkflow().GetEnumerator(), null, (s, a) =>
            {
                if (a.Error != null)
                {
                    Debug.WriteLine("{0}\n{1}", a.Error.Message, a.Error.StackTrace);
                    ShowMapServiceNotFound = true;
                    IsBusy = false;
                }
            });
        }

        private IEnumerable<IResult> GetMapServiceWorkflow()
        {
            IsBusy = true;
            ShowMapServiceNotFound = false;
            var getMapService = new GetMapService(Url);
            yield return getMapService;

            if (getMapService.Result.SingleFusedMapCache && getMapService.Result.SpatialReference.WellKnownId != MapHandler.MapSpatialReference.WellKnownId)
            {
                IsTiledMapServiceInCorrectProjection = false;
                LoadedMapService = null;
                IsBusy = false;
                yield break;
            }

            IsBasemapVisible = getMapService.Result.SingleFusedMapCache;
            LoadedMapService = getMapService.Result;
            IsTiledMapServiceInCorrectProjection = true;
            Title = LoadedMapService.DocumentInfo.Title;
            Description = LoadedMapService.ServiceDescription;
            CopyrightText = LoadedMapService.CopyrightText;

            var getMapImage = new GetMapServicePreview(Url, LoadedMapService);
            yield return getMapImage;
            try
            {
                var image = new BitmapImage();
                using (var ms = new MemoryStream(getMapImage.Result))
                {
                    image.SetSource(ms);
                }

                ImageUrl = image;
                IsImageVisible = true;
            } //Catching an HRESULT Catastropic failure that sometimes occurs on image.setsource
            catch (Exception)
            {
                IsImageVisible = false;
            }

            IsBusy = false;
            yield break;
        }

        public BitmapImage ImageUrl
        {
            get
            {
                return _imageUrl;
            }
            set
            {
                _imageUrl = value;
                NotifyOfPropertyChange(() => ImageUrl);
            }
        }

        public bool IsBusy
        {
            get
            {
                return _isBusy;
            }
            set
            {
                _isBusy = value;
                NotifyOfPropertyChange(() => IsBusy);
            }
        }

        private MapServiceDescription _loadedMapService;
        private bool _isBusy;
        private BitmapImage _imageUrl;

        public MapServiceDescription LoadedMapService
        {
            get { return _loadedMapService; }
            set { _loadedMapService = value; NotifyOfPropertyChange(() => LoadedMapService); }
        }

        public void AddToMap()
        {
            MapHandler.AddMapservice(Url, LoadedMapService, true,null,SaveLayer,IsBasemap);
            Dialog.Close();
        }

        public void Cancel()
        {
            Dialog.Close();
        }
    }
}
