﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using FDOT.GIS.Client.Controls;
using FDOT.GIS.Client.Core;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Icon;
using FDOT.GIS.Client.ImageManipulation;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.Toc
{
    public class LocalFilesViewModel : ObservableBase
    {
        private int _graphicsCount;
        public int GraphicsCount
        {
            get { return _graphicsCount; }
            set
            {
                _graphicsCount = value;
                NotifyOfPropertyChange(() => GraphicsCount);
            }
        }

        private Color _currentDrawColor = SymbolManager.DefaultColor;
        private bool _isBusy;

        private bool _zoomToExtent;

        public bool ZoomToExtent
        {
            get { return _zoomToExtent; }
            set { _zoomToExtent = value; NotifyOfPropertyChange(()=>ZoomToExtent);}
        }

        public string CurrentDrawColor
        {
            get { return ColorConverter.ColorHex(_currentDrawColor); }
            set
            {
                _currentDrawColor = ColorConverter.HexColor(value);
                NotifyOfPropertyChange(() => CurrentDrawColor);
            }
        }

        public IEnumerable<IconCategory> IconCategories
        {
            get { return IconManager.Instance.IconCategories; }
        }

        private string _selectedUrl;
        public string SelectedUrl
        {
            get { return _selectedUrl; }
            set { _selectedUrl = value; NotifyOfPropertyChange(() => SelectedUrl); }
        }

        private IconCategory _selectedIconCategory;
        public IconCategory SelectedIconCategory
        {
            get { return _selectedIconCategory; }
            set { _selectedIconCategory = value; NotifyOfPropertyChange(() => SelectedIconCategory); }
        }

        private bool _isColorSelected = true;
        public bool IsColorSelected
        {
            get { return _isColorSelected; }
            set { _isColorSelected  = value; }
        }

        public bool IsIconSelected { get; set; }

        public ICommand LoadShapeFile
        {
            get
            {
                return new DelegateCommand(o =>
                {
                    // Create an instance of the open file dialog box.
                    var openFileDialog1 = new OpenFileDialog
                    {
                        Filter =
                            "Shape Files(*.shp, *.dbf, *.prj)|*.shp;*.dbf;*.prj",
                        FilterIndex = 1,
                        Multiselect = true
                    };

                    // Call the ShowDialog method to show the dialog box.
                    bool? userClickedOk = openFileDialog1.ShowDialog();

                    // Process input if the user clicked OK.
                    if (userClickedOk == true)
                    {
                        ShowLoadError = false;
                        bool wasAddAttempted = false;
                        foreach (var file in openFileDialog1.Files)
                        {
                            IsBusy = true;
                            //IsBusyMessage = "Loading " + file.Name;

                            if (file.Extension.ToUpper() == ".DBF" || file.Extension.ToUpper() == ".PRJ")
                                continue;

                            FileInfo file1 = file;
                            FileInfo dbfFile = openFileDialog1.Files.SingleOrDefault(f => f.Name.ToUpper() ==
                                file1.Name.ToUpper().Replace(".SHP", ".DBF"));

                            FileInfo prjFile = openFileDialog1.Files.SingleOrDefault(f => f.Name.ToUpper() ==
                                 file1.Name.ToUpper().Replace(".SHP", ".PRJ"));

                            SymbolDefinition symbolDef = IsIconSelected ? new SymbolDefinition(new BitmapImage(new Uri(SelectedUrl))) :
                                new SymbolDefinition("Circle",CurrentDrawColor);

                            if (dbfFile == null)
                                break;

                            wasAddAttempted = true;
                            MapHandler.AddShapeFile(file.Name, file, dbfFile, prjFile, symbolDef, LoadingSuccessful,ZoomToExtent);
                        }

                        if(!wasAddAttempted)
                        {
                            IsBusy = false;
                            ShowLoadError = true;
                        }
                    }
                });
            }
        }

        private bool _showLoadError;
        public bool ShowLoadError
        {
            get { return _showLoadError; }
            set { _showLoadError = value;
                NotifyOfPropertyChange(()=>ShowLoadError); }
        }

        public bool IsBusy
        {
            get
            {
                return _isBusy;
            }
            set
            {
                _isBusy = value; NotifyOfPropertyChange(() => IsBusy);
            }
        }

        public void DrawColorChanged(ColorChangedEventArgs e)
        { _currentDrawColor = e.NewColor; }

        public GenericDialogViewModel Dialog { get; set; }

        public void Cancel()
        {
            Dialog.Close();
        }

        private void LoadingSuccessful()
        {
            Dialog.Close();
        }
    }
}
