﻿using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Persistence;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components.ViewModels.Toc
{
    public class TocSettingsViewModel : SettingsViewModel
    {
        private const string Showaddshapefile = "ShowAddShapeFile";
        private const string Zoomtokmlonload = "ZoomToKmlOnLoad";
        private const string Showaddmapservice = "ShowAddMapService";
        private const string Showaddkmlfile = "ShowAddKmlFile";

        private static readonly TocSettingsViewModel _instance = new TocSettingsViewModel();

        private TocSettingsViewModel()
        {
            
        }

        [Hidden]
        public static TocSettingsViewModel Instance
        {
            get
            {
                return _instance;
            }
        }


        [SwitchButton]
        [DisplayName("Show Add Kml File")]
        [ToolTipText("Allows the addition of kml files to the map.")]
        public bool ShowAddKmlFile
        {
            get { return StateManager.GetProperty<bool>(Showaddkmlfile); }
            set { StateManager.SetProperty(Showaddkmlfile, value);
            NotifyOfPropertyChange(()=>ShowAddKmlFile);}
        }

        [SwitchButton]
        [DisplayName("Show Add Map Service")]
        [ToolTipText("Allows the addition of map services to the map.")]
        public bool ShowAddMapService
        {
            get { return StateManager.GetProperty<bool>(Showaddmapservice); }
            set { StateManager.SetProperty(Showaddmapservice, value);
            NotifyOfPropertyChange(() => ShowAddMapService);
            }
        }

        [SwitchButton]
        [DisplayName("Zoom to Kml Features")]
        [ToolTipText("Zoom to Kml Features on the loading of the kml file.")]
        public bool ZoomToKmlOnLoad
        {
            get { return StateManager.GetProperty<bool>(Zoomtokmlonload); }
            set
            {
                StateManager.SetProperty(Zoomtokmlonload, value);
                NotifyOfPropertyChange(() => ZoomToKmlOnLoad);
            }
        }

        [SwitchButton]
        [DisplayName("Show Add Shape File")]
        [ToolTipText("Allows the addition of shape files to the map.")]
        public bool ShowAddShapeFile
        {
            get { return StateManager.GetProperty<bool>(Showaddshapefile); }
            set { StateManager.SetProperty(Showaddshapefile, value);
            NotifyOfPropertyChange(() => ShowAddShapeFile);
            }
        }
    }
}
