﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Windows;
using Caliburn.Micro;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.DotCodes;
using FDOT.GIS.DataContracts;
using System;
using System.Windows.Controls;
using System.ComponentModel;
using System.Linq.Expressions;

namespace FDOT.GIS.Client.Components.Views.QueryTool
{
    public partial class QueryCriterionControl : UserControl, INotifyPropertyChanged
    {
        private ObservableCollection<DotCode> _criteriaCodes;
        private bool _isFreeTextCriterion;
        private bool _isDateTimeCriterion;

        public static readonly DependencyProperty FeatureTypeProperty =
        DependencyProperty.Register("FeatureType", typeof(FeatureType), typeof(QueryCriterionControl), new PropertyMetadata(null));

        public FeatureType FeatureType
        {
            get { return (FeatureType)GetValue(FeatureTypeProperty); }
            set { SetValue(FeatureTypeProperty, value); }
        }


        public static readonly DependencyProperty CriteriaProperty =
        DependencyProperty.Register("Criteria", typeof(QueryCriteria), typeof(QueryCriterionControl), new PropertyMetadata(null));

        public QueryCriteria Criteria
        {
            get { return (QueryCriteria)GetValue(CriteriaProperty); }
            set { SetValue(CriteriaProperty, value); }
        }

        public ObservableCollection<DotCode> CriteriaCodes
        {
            get { return _criteriaCodes; }
            set
            {
                if (value != null && value.Count == 0)
                {
                    _criteriaCodes = null;
                }
                else
                {
                    _criteriaCodes = value;
                }
                IsFreeTextCriterion = _criteriaCodes == null ? true : false;
                NotifyOfPropertyChange(() => CriteriaCodes);
            }
        }


        public bool IsFreeTextCriterion
        {
            get { return _isFreeTextCriterion; }
            set
            {
                _isFreeTextCriterion = value;
                NotifyOfPropertyChange(() => IsFreeTextCriterion);
            }
        }


        public bool IsDateTimeCriterion
        {
            get { return _isDateTimeCriterion; }
            set
            {
                _isDateTimeCriterion = value;
                if (_isDateTimeCriterion)
                { IsFreeTextCriterion = false; }
                NotifyOfPropertyChange(() => IsDateTimeCriterion);
            }
        }

        public void Initialize(FeatureType featureType, QueryCriteria criteria)
        {
            FeatureType = featureType;
            Criteria = criteria;
        }


        public void CriteriaSelectionChanged(object sender, RoutedEventArgs e)
        {
            if (Criteria != null && Criteria.Attribute != null && !string.IsNullOrEmpty(Criteria.Attribute.Name))
            {
                if (Criteria != null && Criteria.Attribute != null)
                { IsDateTimeCriterion = Criteria.Attribute.Type == typeof(DateTime); }

                if (!IsDateTimeCriterion)
                { Coroutine.BeginExecute(DoWorkFlow().GetEnumerator(), null, Complete); }
            }
        }

        public void CriterionChanged(object sender, RoutedEventArgs e)
        {
            UpdateSource(sender);
            EventAggregatorManager.Events.Publish(new CriterionChangedEvent(Criteria));
        }


        public static void UpdateSource(object sender)
        {
            var textBox = sender as TextBox;
            if (textBox != null)
            {
                // If a TextBox has the focus, this forces it to update its source.
                // It is needed when using Enter and Escape to activate Save and Cancel button,
                // because the focused TextBox does not update in Silverlight, even when
                // a call is made to ButtonSave.Focus() or ButtonCancel.Focus().
                TextBox tb = System.Windows.Input.FocusManager.GetFocusedElement() as TextBox;
                if (tb != null)
                    tb.GetBindingExpression(TextBox.TextProperty).UpdateSource();
            }
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private IEnumerable<IResult> DoWorkFlow()
        {
            yield return CodeProviderManager.Instance.GetProvider(Criteria.Attribute, this, "CriteriaCodes");
        }



        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyOfPropertyChange(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }

        public virtual void NotifyOfPropertyChange<TProperty>(Expression<Func<TProperty>> property)
        {
            NotifyOfPropertyChange(property.GetMemberInfo().Name);
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }
    }

    public class CriterionChangedEvent
    {
        public QueryCriteria Criteria { get; private set; }

        public CriterionChangedEvent(QueryCriteria criteria)
        {
            Criteria = criteria;
        }
    }

}