﻿using System;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Media;
using Caliburn.Micro;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.ViewModels;

namespace FDOT.GIS.Client.Components
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class ZoomToBoxComponent : ComponentBase
    {
        private GeometrySelectionService _gss;

        public override DependencyObject HelpView
        {
            get
            {
                var help = new ZoomToBoxHelpView();
                View.SetModel(help, new ZoomToBoxHelpViewModel());
                return help;
            }
        }

        protected override void OnExecute()
        {
            SymbolManager.SetFillSymbol("ZoomToBoxFill");
            _gss = new GeometrySelectionService(new GeometrySelectionServiceConfiguration {Color = Colors.Black });
            _gss.DrawCanceled += gss_DrawCanceled;
            _gss.SelectGeometry(GeometryType.Box, ZoomToGeometry);
        }

        void gss_DrawCanceled(object sender, EventArgs e)
        {
            SymbolManager.SetDefaultPalette();
            ToolbarViewModel.Instance.InactivateComponent(typeof(ZoomToBoxComponent));
            if(_gss!=null)
            {_gss.DrawCanceled -= gss_DrawCanceled;}
            _gss = null;
        }

        protected void ZoomToGeometry(DataContracts.Geometry geometry)
        {
            SymbolManager.SetDefaultPalette();
            MapHandler.ZoomToGeometry(geometry);
            
            ToolbarViewModel.Instance.InactivateComponent(typeof(ZoomToBoxComponent));
            if(_gss != null)
            {_gss.DrawCanceled -= gss_DrawCanceled;}
            _gss = null;
        }
        protected override string IconResourceUri
        {
            get { return "Assets/Images/zoom_in.png"; }
        }
    }
}