﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using Caliburn.Micro;
using FDOT.GIS.Client.Commands;
using FDOT.GIS.Client.Components.ViewModels.Help;
using FDOT.GIS.Client.Components.Views.Help;
using FDOT.GIS.Client.Domain;
using FDOT.GIS.Client.Domain.Gis;
using FDOT.GIS.Client.Domain.Graphics;
using FDOT.GIS.Client.ViewModels;
using FDOT.GIS.DataContracts;

namespace FDOT.GIS.Client.Components
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [DisplayName("Zoom To Coordinates")]
    public class ZoomToCoordinatesComponent : ComponentBase
    {
        private GenericDialogViewModel _dialog;
        private GraphicsManager _graphicsManager;
        public double Latitude { get; set; }
        public double Longitude { get; set; }

        [Hidden]
        public override DependencyObject HelpView
        {
            get
            {
                var help = new ZoomToCoordinatesHelpView();
                View.SetModel(help, new ZoomToCoordinatesHelpViewModel());
                return help;
            }
        }

        protected override void OnExecute()
        {
            _dialog = CreateChildWindow(this);
            _graphicsManager = GraphicManagerFactory.CreateGraphicsManager();
            _dialog.Closed += DialogClosed;
        }

        void DialogClosed(object sender, EventArgs e)
        {
            _graphicsManager.Dispose();
            _dialog.Closed -= DialogClosed;
            _dialog = null;
        }

        protected override string IconResourceUri
        {
            get { return "Assets/Images/zoom_to_coordinates.png"; }
        }

        /// <summary>
        /// Does the projection and zooming to specified lat, long
        /// </summary>
        /// <returns></returns>
        private IEnumerable<IResult> GetProjectionWorkflow()
        {
            var point = new DataContracts.Geometry { X=Longitude, Y=Latitude};
            var project = new Project(new SpatialReference {WellKnownId = WellKnownIds.WGS1984},
                                      MapHandler.MapSpatialReference, point);
            
            yield return project;

            MapHandler.ZoomToGeometry(project.Result.First());
            _graphicsManager.ClearGraphics();
            _graphicsManager.AddGraphic(project.Result.First(), new SymbolDefinition("RotatingBlueMarkerSymbol"), null, null);

            yield break;
        }

        /// <summary>
        /// Executes the workflow to perform the zoom to the specified lat, long
        /// </summary>
        public void ZoomToCoordinates()
        {
            var executionContext = new ActionExecutionContext();
            Coroutine.BeginExecute(GetProjectionWorkflow().GetEnumerator(), executionContext, Complete);
        }

        private static void Complete(object sender, ResultCompletionEventArgs e)
        {
            if(e.Error!=null)
                throw e.Error;
        }
    }
}